﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class TreePlacementBehaviour : MonoBehaviour 
{
    public GameObject treePrefab;

    public int poolSize;

    public List<GameObject> treePool;

    public int growIndex = 0;
    public float growthTime = float.NegativeInfinity;

    public float rotationAngle = 20f;

    public float speed = 0.5f;

    Transform _refTrans;
    Transform _transform;
    Rigidbody _rigidBody;

    int fallHash, growHash;
    
    private CameraTransitionBehaviour cameraTransition;
    public AnimationCurve camTransitionCurve;
    public float maxCamTransitionTime = 1f;

    public LayerMask layerMask;

    RaycastHit rch;
    public Texture2D texture;
    public float height = 14.9f;

    public int currentNote = 0;
    public AudioClip[] notes;

    TreeUIBehaviour ui;
    
    public float treefallSpeed;
    public float treefallSpeedIncrease;

    private int treeFallState = -1;

    public GameObject goal;
	// Use this for initialization

    void Awake()
    {
        goal.SetActive(false);
    }
	IEnumerator Start () 
    {
        ui = GameObject.FindObjectOfType<TreeUIBehaviour>();

        fallHash = Animator.StringToHash("Fall");
        growHash = Animator.StringToHash("Grow");

        _transform = transform;
        _rigidBody = GetComponent<Rigidbody>();

        GameObject refGO = new GameObject("_RigidRef");
        _refTrans = refGO.transform;
        _refTrans.position = _transform.position;
        _refTrans.rotation = _transform.rotation;

        cameraTransition = FindObjectOfType<CameraTransitionBehaviour>();

        treePool = new List<GameObject>(poolSize);

        for (int i = 0; i < poolSize; i++)
        {
            GameObject go = Instantiate(treePrefab);

            TreeBehaviour tree = go.GetComponent<TreeBehaviour>();
            tree.Setup(i, this);

            go.gameObject.SetActive(false);
            go.name = "Tree " + i;
            treePool.Add(go);

            if (i % 30 == 29)
                yield return 0;
        }

        growthTime = 0f;
	}

    void FixedUpdate()
    {
        if (float.IsNegativeInfinity(growthTime))
            return;

        _refTrans.position += _transform.forward * Time.fixedDeltaTime * speed;

        // Cast a ray to see where on the box it hit the ground?
        if (Physics.Raycast(_refTrans.position + Vector3.up * 10f, Vector3.down, out rch, 1000f, layerMask.value))
        {
            float diff = Mathf.Abs(_refTrans.position.y - rch.point.y);
            if (diff > 1f)
            {
                enabled = false;
                ui.ShowGameOver(false);
                return;
            }

            Vector3 v = _refTrans.position;
            v.y = rch.point.y;
            _refTrans.position = v;
        }
        else
        {
            Debug.Log("YOU HAVE DIED");
            ui.ShowGameOver(false);
            enabled = false;
        }

        if (Input.GetMouseButton(1) || Input.GetKey(KeyCode.RightArrow) || Input.GetKey(KeyCode.D))
        {
            _refTrans.Rotate(0, speed *  rotationAngle * Time.fixedDeltaTime, 0);
        }
        else if (Input.GetMouseButton(0) || Input.GetKey(KeyCode.LeftArrow) || Input.GetKey(KeyCode.A))
        {
            _refTrans.Rotate(0, speed * -rotationAngle * Time.fixedDeltaTime, 0);
        }

        _rigidBody.MovePosition(_refTrans.position);
        _rigidBody.MoveRotation(_refTrans.rotation);
    }


	// Update is called once per frame
	void Update () 
    {
        growthTime += Time.deltaTime;

        if (growthTime >= 1.5f / speed)
        {
            GameObject go = treePool[growIndex % poolSize];

            if (go.activeSelf)
                go.SetActive(false);

            Transform t = go.transform;
            t.position = _transform.position - _transform.forward;

            if (Physics.Raycast(t.position + Vector3.up * 10f, Vector3.down, out rch, 1000f, layerMask.value))
            {
                Vector3 v = t.position;
                v.y = rch.point.y;
                t.position = v;
            }

            t.rotation = _transform.rotation;
            t.Rotate(270, 270, 0);


            go.SetActive(true);

            growIndex++;

            growthTime = 0;

            treeFallState = ui.TreePlanted(growIndex);
            if (treeFallState == 0)
                StartCoroutine(BeginFalling());
        }
	}

    IEnumerator BeginFalling()
    {
        ui.dangerText.SetActive(true);

        bool fallen = false;

        float t = 0f;

        cameraTransition.state = 0f;

        while (t < maxCamTransitionTime)
        {
            float p = camTransitionCurve.Evaluate(t);
            cameraTransition.state = p;
            t += Time.deltaTime;

            if (!fallen && p >= 0.99f)
            {
                Animator anim = treePool[0].GetComponent<Animator>();
                anim.speed = treefallSpeed;
                anim.Play(fallHash, 0, 0f);
                fallen = true;
            }

            yield return 0;
        }

        cameraTransition.state = 0;
    }

    internal void MakeNextFall(int index)
    {
        if (!enabled)
            return;

        int i = (index + 1) % poolSize;
        if (treePool[i].activeSelf)
        {
            Animator anim = treePool[i].GetComponent<Animator>();
            anim.speed = treefallSpeed;

            if (treeFallState > 0)
                treefallSpeed += treefallSpeedIncrease;

            anim.Play(fallHash, 0, 0f);
            //treePool[i].GetComponent<AudioSource>().pitch /= 1.10f;
            
            AudioSource ss = treePool[i].GetComponent<AudioSource>();
            ss.pitch = 0.9f;
            ss.volume = 0.35f;
            ss.Play();
        }
        else
        {
            // TODO:  Game Over Event.
            enabled = false;
            ui.ShowGameOver(false);
        }
    }

    void OnTriggerEnter(Collider c)
    {
        if (c.transform.parent != null)
            Debug.Log("Collided with " + c.transform.parent.name + ".");

        if (c.gameObject.CompareTag("Seed"))
        {
            c.GetComponent<SeedBehaviour>().Collected(this);
            c.gameObject.SetActive(false);
            if (ui.CollectSeed())
            {
                goal.gameObject.SetActive(true);
                ui.goalOpenText.SetActive(true);
            }

        }
        else if (c.gameObject.CompareTag("Goal"))
        {
            ui.ShowGameOver(true);
            enabled = false;
        }
        else if (c.gameObject.CompareTag("Tree"))
        {
            ui.ShowGameOver(false);
            enabled = false;
        }
    }
}
