﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class TreeUIBehaviour : MonoBehaviour {

    public RectTransform arrowRectTransform = null;

    public GameObject dangerText = null;
    public GameObject goalOpenText = null;

    public Text seedText = null;

    private int seedTotal = 0;
    private int seedsCollected;

    public int maxTrees = 10;
    public int dangerTrees = 10;
    int treesGrown = 0;
    int totalTrees;

    public AudioClip winSFX, loseSFX;

    public RectTransform[] gradientImageRects;

    public Vector2 arrowRange;

    public CanvasGroup resultGroup = null;

    public Text gameoverText;

	// Use this for initialization
	void Start () 
    {
        resultGroup.interactable = false;
        resultGroup.alpha = 0f;

        seedTotal = GameObject.FindGameObjectsWithTag("Seed").Length;
        seedsCollected = 0;
        seedText.text = string.Format("Seeds Collected:  {0}/{1}", seedsCollected, seedTotal);

        totalTrees = maxTrees + dangerTrees;

        float p1 = Mathf.InverseLerp(0, totalTrees, maxTrees);
        float p2 = Mathf.InverseLerp(0, totalTrees, dangerTrees);

        gradientImageRects[0].anchorMax = new Vector2(Mathf.InverseLerp(0, totalTrees, maxTrees), 1f);
        gradientImageRects[1].anchorMin = new Vector2(gradientImageRects[0].anchorMax.x, 0f);

	}

    public bool CollectSeed()
    {
        seedsCollected++;

        seedText.text = string.Format("Seeds Collected:  {0}/{1}", seedsCollected, seedTotal);

        return seedsCollected == seedTotal;
    }

    public void ShowGameOver(bool win)
    {
        AudioSource ss = GetComponent<AudioSource>();
        ss.clip = win ? winSFX : loseSFX;
        ss.Play();

        if (!win)
        {
            gameoverText.text = "Game Over";
        }
        else
        {
            gameoverText.text = string.Format("Congratulations!\nYou planted {0} Grominows!", treesGrown);
        }

        StartCoroutine(FadeInResults(1));
    }

    public void ReplayLevel()
    {
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    public void NextLevel()
    {
        int nextIndex = (SceneManager.GetActiveScene().buildIndex + 1) % SceneManager.sceneCountInBuildSettings;
        SceneManager.LoadScene(nextIndex);
    }


    IEnumerator FadeInResults(float t)
    {
        float time = 0f;
        while (time < t)
        {
            time += Time.deltaTime;
            resultGroup.alpha = Mathf.Pow(time / t, 2);
            yield return 0;
        }

        resultGroup.alpha = 1f;
        resultGroup.interactable = true;
    }

    public int TreePlanted(int count)
    {
        treesGrown = count;
        Vector3 v = arrowRectTransform.localPosition;

        float p = (float)count / (float)totalTrees;

        v.x = Mathf.Lerp(arrowRange.x, arrowRange.y, p);
        arrowRectTransform.localPosition = v;

        if (maxTrees == count)
            return 0;
        if (count >= dangerTrees)
            return 1;
        else
            return -1;
    }
}
