﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

[AddComponentMenu("UI/Effects/UI Gradient")]
public partial class UIGradient : BaseMeshEffect
{
    [SerializeField]
    private Gradient gradient = null;

    [SerializeField()]
    private bool leftToRight = false;

    /*[SerializeField()]
    AnimationCurve curve;

    [SerializeField()]
    float curveStrength;*/
    public override void ModifyMesh(VertexHelper vh)
    {
        //throw new System.NotImplementedException();
    }

    public override void ModifyMesh(Mesh mesh)
    {
        if (!this.IsActive())
            return;

        List<UIVertex> list = new List<UIVertex>();
        using (VertexHelper vertexHelper = new VertexHelper(mesh))
        {
            vertexHelper.GetUIVertexStream(list);
        }

        ModifyVertices(list);  // calls the old ModifyVertices which was used on pre 5.2

        using (VertexHelper vertexHelper2 = new VertexHelper())
        {
            vertexHelper2.AddUIVertexTriangleStream(list);
            vertexHelper2.FillMesh(mesh);
        }
    }
    public void ModifyVertices(List<UIVertex> vertexList)
    {
        if (!IsActive())
        {
            return;
        }

        if (!leftToRight)
        {
            int count = vertexList.Count;
            float bottomY = vertexList[0].position.y;
            float topY = vertexList[0].position.y;

            for (int i = 1; i < count; i++)
            {
                float y = vertexList[i].position.y;
                if (y > topY)
                {
                    topY = y;
                }
                else if (y < bottomY)
                {
                    bottomY = y;
                }
            }

            float uiElementHeight = topY - bottomY;

            for (int i = 0; i < count; i++)
            {
                UIVertex uiVertex = vertexList[i];
                uiVertex.color *= gradient.Evaluate((uiVertex.position.y - bottomY) / uiElementHeight);
                vertexList[i] = uiVertex;
            }
        }
        else
        {
            int count = vertexList.Count;
            float bottomY = vertexList[0].position.x;
            float topY = vertexList[0].position.x;

            for (int i = 1; i < count; i++)
            {
                float x = vertexList[i].position.x;
                if (x > topY)
                {
                    topY = x;
                }
                else if (x < bottomY)
                {
                    bottomY = x;
                }
            }

            float uiElementHeight = topY - bottomY;

            for (int i = 0; i < count; i++)
            {
                UIVertex uiVertex = vertexList[i];
                uiVertex.color *= gradient.Evaluate((uiVertex.position.x - bottomY) / uiElementHeight);

                //uiVertex.position += curveStrength * Vector3.up * curve.Evaluate((uiVertex.position.x - bottomY) / uiElementHeight);
                vertexList[i] = uiVertex;
            }
        }
    }
}